#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Callable, Optional, Union

from torch import Tensor

from aac_metrics.functional.coco_cider_d import coco_cider_d
from aac_metrics.functional.coco_spice import coco_spice


def spider(
    candidates: list[str],
    mult_references: list[list[str]],
    return_all_scores: bool = True,
    # CIDEr args
    n: int = 4,
    sigma: float = 6.0,
    tokenizer: Callable[[str], list[str]] = str.split,
    return_tfidf: bool = False,
    # SPICE args
    java_path: str = "java",
    tmp_path: str = "/tmp",
    cache_path: str = "$HOME/aac-metrics-cache",
    n_threads: Optional[int] = None,
    java_max_memory: str = "8G",
    verbose: int = 0,
) -> Union[Tensor, tuple[dict[str, Tensor], dict[str, Tensor]]]:
    if len(candidates) != len(mult_references):
        raise ValueError(
            f"Number of candidates and mult_references are different (found {len(candidates)} != {len(mult_references)})."
        )

    cider_d_out = coco_cider_d(
        candidates,
        mult_references,
        return_all_scores,
        n=n,
        sigma=sigma,
        tokenizer=tokenizer,
        return_tfidf=return_tfidf,
    )
    spice_out = coco_spice(
        candidates,
        mult_references,
        return_all_scores,
        java_path=java_path,
        tmp_path=tmp_path,
        cache_path=cache_path,
        n_threads=n_threads,
        java_max_memory=java_max_memory,
        verbose=verbose,
    )

    if return_all_scores:
        assert isinstance(cider_d_out, tuple)
        assert isinstance(spice_out, tuple)
        cider_d_global_scores, cider_d_local_scores = cider_d_out
        spice_global_scores, spice_local_scores = spice_out

        spider_global_scores = {
            "cider_d": cider_d_global_scores["cider_d"],
            "spice": spice_global_scores["spice"],
            "spider": (cider_d_global_scores["cider_d"] + spice_global_scores["spice"])
            / 2.0,
        }
        spider_local_scores = {
            "cider_d": cider_d_local_scores["cider_d"],
            "spice": spice_local_scores["spice"],
            "spider": (cider_d_local_scores["cider_d"] + spice_local_scores["spice"])
            / 2.0,
        }
        return spider_global_scores, spider_local_scores
    else:
        assert isinstance(cider_d_out, Tensor)
        assert isinstance(spice_out, Tensor)
        return (cider_d_out + spice_out) / 2.0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Callable, Union

from torch import Tensor

from aac_metrics.classes.base import Metric
from aac_metrics.functional.coco_rouge_l import (
    _coco_rouge_l_compute,
    _coco_rouge_l_update,
)


class CocoRougeL(Metric):
    full_state_update = False
    higher_is_better = True
    is_differentiable = False

    min_value = 0.0
    max_value = 1.0
    is_linear = True

    def __init__(
        self,
        return_all_scores: bool = True,
        beta: float = 1.2,
        tokenizer: Callable[[str], list[str]] = str.split,
    ) -> None:
        super().__init__()
        self._return_all_scores = return_all_scores
        self._beta = beta
        self._tokenizer = tokenizer

        self._rouge_l_scores = []

    def compute(self) -> Union[Tensor, tuple[dict[str, Tensor], dict[str, Tensor]]]:
        return _coco_rouge_l_compute(
            self._rouge_l_scores,
            self._return_all_scores,
        )

    def reset(self) -> None:
        self._rouge_l_scores = []
        return super().reset()

    def update(
        self,
        candidates: list[str],
        mult_references: list[list[str]],
    ) -> None:
        self._rouge_l_scores = _coco_rouge_l_update(
            candidates,
            mult_references,
            self._beta,
            self._tokenizer,
            self._rouge_l_scores,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

from typing import Optional, Union

from torch import Tensor

from aac_metrics.classes.base import Metric
from aac_metrics.functional.coco_spice import coco_spice


logger = logging.getLogger(__name__)


class CocoSPICE(Metric):
    full_state_update = False
    higher_is_better = True
    is_differentiable = False

    min_value = 0.0
    max_value = 1.0
    is_linear = None

    def __init__(
        self,
        return_all_scores: bool = True,
        java_path: str = "java",
        tmp_path: str = "/tmp",
        cache_path: str = "$HOME/aac-metrics-cache",
        n_threads: Optional[int] = None,
        java_max_memory: str = "8G",
        verbose: int = 0,
    ) -> None:
        super().__init__()
        self._return_all_scores = return_all_scores
        self._java_path = java_path
        self._tmp_path = tmp_path
        self._cache_path = cache_path
        self._n_threads = n_threads
        self._java_max_memory = java_max_memory
        self._verbose = verbose

        self._candidates = []
        self._mult_references = []

    def compute(self) -> Union[Tensor, tuple[dict[str, Tensor], dict[str, Tensor]]]:
        return coco_spice(
            self._candidates,
            self._mult_references,
            self._return_all_scores,
            self._java_path,
            self._tmp_path,
            self._cache_path,
            self._n_threads,
            self._java_max_memory,
            self._verbose,
        )

    def reset(self) -> None:
        self._candidates = []
        self._mult_references = []
        return super().reset()

    def update(
        self,
        candidates: list[str],
        mult_references: list[list[str]],
    ) -> None:
        self._candidates += candidates
        self._mult_references += mult_references



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Callable, Optional, Union

from torch import Tensor

from aac_metrics.functional.mult_cands import mult_cands_metric
from aac_metrics.functional.spider import spider


def spider_max(
    mult_candidates: list[list[str]],
    mult_references: list[list[str]],
    return_all_scores: bool = True,
    # CIDEr args
    n: int = 4,
    sigma: float = 6.0,
    tokenizer: Callable[[str], list[str]] = str.split,
    return_tfidf: bool = False,
    # SPICE args
    java_path: str = "java",
    tmp_path: str = "/tmp",
    cache_path: str = "$HOME/aac-metrics-cache",
    n_threads: Optional[int] = None,
    java_max_memory: str = "8G",
    verbose: int = 0,
) -> Union[Tensor, tuple[dict[str, Tensor], dict[str, Tensor]]]:
    return mult_cands_metric(
        spider,
        "spider",
        mult_candidates,
        mult_references,
        return_all_scores,
        "max",
        # CIDEr args
        n=n,
        sigma=sigma,
        tokenizer=tokenizer,
        return_tfidf=return_tfidf,
        # SPICE args
        java_path=java_path,
        tmp_path=tmp_path,
        cache_path=cache_path,
        n_threads=n_threads,
        java_max_memory=java_max_memory,
        verbose=verbose,
    )
